@@ -10,16 +10,18 @@ data_rec <- mdcev.data(data_rec, subset = id < 100,
 					   alt.var = "alt",
 					   choice = "quant")
 
-result <- mdcev( ~ alt -1,
+result <- mdcev( ~ alt-1,
 				   data = data_rec,
 				   model = "hybrid0",
+				 psi_ascs = 0,
 				   algorithm = "MLE",
+				 std_errors = "mvn",
 				   print_iterations = FALSE)
 
 nalts <- result$stan_data[["J"]]
 model_num <- 1
 npols <- 2
-policies<-	CreateBlankPolicies(npols, nalts, result$stan_data[["dat_psi"]], price_change_only = TRUE)
+policies<-	CreateBlankPolicies(npols, result, price_change_only = TRUE)
 
 df_sim <- PrepareSimulationData(result, policies, nsims = 3)
 income <- df_sim[["df_indiv"]][["income"]][[1]]
@@ -32,6 +34,7 @@ price_j <- price[-1]
 quant_num <- income - quant_j  %*% price[-1]
 quant <- c(quant_num, quant_j)
 psi_j <- result[["stan_data"]][["dat_psi"]][1:nalts,] %*% t(result[["stan_fit"]][["par"]][["psi"]])
+phi_j <- rep(0,nalts)
 gamma_j <- result[["stan_fit"]][["par"]][["gamma"]]
 gamma <- c(1, gamma_j)
 alpha <- rep(1e-03, nalts+1)
@@ -45,19 +48,26 @@ tol_e <- 1e-20
 tol_l <- 1e-20
 max_loop = 999
 
+PRNG <-rstan::get_rng(seed = 3)
+o <- rstan::get_stream()
+
 error <- DrawError_rng(quant_num, quant_j, price[-1],
-				  psi_j, gamma_j, alpha, scale,
-				  nalts = nalts, nerrs = 2, cond_error = 1, draw_mlhs = 1)
+				  psi_j, phi_j, gamma_j, alpha, scale,
+				  model_num = model_num, nalts = nalts, nerrs = 2, cond_error = 1, draw_mlhs = 1,
+				  PRNG, o)
 
 	psi_b_err <- exp(c(0, psi_j) + error[[1]])
 	MUzero_b <- psi_b_err / price
 #	 Test hybrid algo
-	mdemand <- MarshallianDemand(income, price, MUzero_b, gamma, alpha,
-								 nalts, algo_gen = 0, tol_e = tol_e, max_loop = max_loop)
+	algo_gen = 0
+	mdemand <- MarshallianDemand(income, price, MUzero_b, c(1, phi_j), gamma, alpha,
+								 nalts, algo_gen, model_num, tol_e, max_loop, o)
+
 	expect_true(sum(abs(mdemand - quant)) < tol)
 	# Test general algo
-	mdemand <- MarshallianDemand(income, price, MUzero_b, gamma, alpha,
-								 nalts, algo_gen = 1, tol_e = tol_e, max_loop = max_loop)
+	mdemand <- MarshallianDemand(income, price, MUzero_b, c(1, phi_j), gamma, alpha,
+								 nalts, algo_gen = 1, model_num, tol_e = tol_e, max_loop = max_loop, o)
+
 	expect_true(sum(abs(mdemand - quant)) < tol)
 
 error <- c(0.0000000000, -0.77612399, -0.55169780, -0.02143232, -0.81241994, -0.19768961,
@@ -66,25 +76,25 @@ error <- c(0.0000000000, -0.77612399, -0.55169780, -0.02143232, -0.81241994, -0.
 
 psi_b_err <- exp(c(0, psi_j) + error)
 MUzero_b <- psi_b_err / price
-mdemand <- MarshallianDemand(income, price, MUzero_b, gamma, alpha,
-							 nalts, algo_gen = 0, tol_e = tol_e, max_loop = max_loop);
+mdemand <- MarshallianDemand(income, price, MUzero_b, c(1, phi_j), gamma, alpha,
+							 nalts, algo_gen = 0, model_num, tol_e = tol_e, max_loop = max_loop, o)
 
 	util <- ComputeUtilJ(income, mdemand[-1], price[-1],
-							 psi_b_err[-1], gamma[-1], alpha,
-							 nalts, model_num)
+							 psi_b_err[-1], phi_j, gamma[-1], alpha,
+							 nalts, model_num, o)
 
 	expect_true(abs(util - 1011.104111) < tol)
 
 	price_p <- price + c(.001,rep(1,nalts))
  	MUzero_p <- psi_b_err / price_p
 
-	hdemand <- HicksianDemand(util, price_p, MUzero_p,  gamma, alpha,
-			nalts, algo_gen = 0, model_num, tol_l = tol_l, max_loop = max_loop)
+	hdemand <- HicksianDemand(util, price_p, MUzero_p,  c(1, phi_j), gamma, alpha,
+							nalts, algo_gen = 0, model_num, tol_l = tol_l, max_loop = max_loop, o)
 	wtp_err <- income - t(price_p) %*% hdemand
 	expect_true(abs(wtp_err - (-62.4995)) < tol)
 
-	hdemand <- HicksianDemand(util, price_p, MUzero_p, gamma, alpha,
-							  nalts, algo_gen = 1, model_num, tol_l = tol_l, max_loop = max_loop)
+	hdemand <- HicksianDemand(util, price_p, MUzero_p, c(1, phi_j), gamma, alpha,
+							  nalts, algo_gen = 1, model_num, tol_l = tol_l, max_loop = max_loop, o)
 	wtp_err <- income - t(price_p) %*% hdemand
 	expect_true(abs(wtp_err - (-62.4995)) < tol)
 
